package org.synbiowave;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.synbiowave.menu.MenuAnswer;
import org.synbiowave.menu.MenuAnswerDeserializer;
import org.synbiowave.menu.MenuCallBack;
import org.synbiowave.menu.MenuItem;
import org.synbiowave.menu.MenuItemFactory;
import org.synbiowave.menu.items.Formular;

import com.google.appengine.repackaged.com.google.common.collect.ImmutableMap;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonParseException;
import com.google.wave.api.AbstractRobot;
import com.google.wave.api.Blip;
import com.google.wave.api.BlipContentRefs;
import com.google.wave.api.ElementType;
import com.google.wave.api.Gadget;
import com.google.wave.api.Wavelet;
import com.google.wave.api.event.Event;
import com.google.wave.api.event.GadgetStateChangedEvent;
import com.google.wave.api.event.WaveletSelfAddedEvent;

public abstract class AbstractSynBioWaveRobot extends AbstractRobot {

	//Final Variables
	private static final long serialVersionUID = -5759256283819622447L;
	protected static final Logger log = Logger.getLogger(AbstractSynBioWaveRobot.class.getName());
	protected final String menuGadgetURL = "http://synbiowave.svn.sourceforge.net/viewvc/synbiowave/trunk/menuGadget/menuGadget.xml";
	
	//Non-final Variables 
	//USE WITH EXTREM CAUTION, AS THEY WILL REGULAR BE SET TO NULL BY APPENGINE
	//MAKE SURE TO REDEFINE THEM BEFORE EVERY USAGE
	private Gson gson = null;
	protected HashMap<String,MenuCallBack> menuCallBacks = new HashMap<String,MenuCallBack>();
	
	
	
	/**
	 * Override this to generate the menu this robot offers.
	 * 
	 * @param menu the menu
	 * @throws Exception 
	 */
	public abstract void createMenu(MenuItem menu, MenuItemFactory mif, Event event) throws Exception;
	
	public MenuItemFactory getMenuItemFactory(Event event){
		MenuItemFactory mif = new MenuItemFactory();
		mif.setRobotID(this.getMyRobotId(event.getWavelet()));
		return mif;}	
	
//	Allocate events.
	
	public String getRobotAdress(Event event){
		return event.getWavelet().getRobotAddress();
	}
	
	public void onSynBioWaveRobotSelfAdded(WaveletSelfAddedEvent event){
		this.addMenuToGadget(event);}

	public void onSynBioWaveGadgetChanged(GadgetStateChangedEvent event) {
		
		Blip blip = event.getBlip();
		Gadget gadget = (Gadget) blip.getElements().get(event.getIndex());
		
		if(gadget.getUrl().contentEquals(this.menuGadgetURL))
		{
			evaluateMenu(event);
			}
		}

	private void evaluateMenu(GadgetStateChangedEvent event) {
		try{
			BlipContentRefs ref=event.getBlip().at(event.getIndex());
			Map<String,String> map = ref.value().asElement().getProperties();
		
			for (Entry<String, String> entry : map.entrySet()){
				String id = entry.getKey().substring(3);
				if ( id.startsWith(this.getMyRobotId(event.getWavelet())) && !entry.getValue().contentEquals("{'event' : 'done'}")) {
					log.info("Menu-Event detected: " + id + " => " + entry.getValue() + "\n");
			
					this.createMenu(new MenuItem("tmp","tmp","tmp"), this.getMenuItemFactory(event), event);
				
					if ( menuCallBacks.containsKey(id) ){
						log.info("Callback for Menu-Event availible");
					
						if (this.gson == null){
							GsonBuilder gb = new GsonBuilder();
							gb.registerTypeAdapter(MenuAnswer.class, new MenuAnswerDeserializer());
							this.gson = gb.serializeNulls().create();}
					
						MenuAnswer ma = gson.fromJson(entry.getValue(), MenuAnswer.class);
						String type = ma.getEvent();
						log.info("Menu-Event is of type " + type + "\n");
					
						if (type.contentEquals("execute") ){
							log.info("Executing Button-Callback with id " + id + ":\n");
							this.menuCallBacks.get(id).onButtonClicked(event, null, this);
							log.info("Button-Callback with id " + id + " executed\n");}
					
						else if (type.contentEquals("changeValue")){
							MenuItem globalMenu = new MenuItem(map.get("ui.structure"));		
							MenuItem tmpItem = globalMenu.getSubItemByID(id);
							MenuItem button = new MenuItem("button","formbutton",id.substring(0,id.length()-1));
							button.appendSubItem(tmpItem);
							Formular answerItem = new Formular(button);
							for ( String key : ma.getOptions().keySet() ){
								answerItem.getSubItemByID(key).setOption("value", ma.getOptions().get(key));}
								this.menuCallBacks.get(id).onFormSubmit(event, answerItem, this);
							log.info("Formular-Callback with id " + id + " executed\n");}
							
						else if (!type.contentEquals("none")){
							throw new Exception("Unknown Menu Event: " + entry.getValue());}
						}
					else log.info("No callback for Menu-Event found");
				
						//mark the menu-event as done
						map.put(entry.getKey(), "{'event' : 'done'}");
						ref.updateElement(map);
						log.info("Menu-Event unset\n");
				}
			}
			HashMap<String,String> tmpmap = new HashMap<String,String>();
			tmpmap.putAll(map);
			tmpmap.put("ui.structure","<removed for readability>");
			tmpmap.put("url","<removed for readability>");
			log.info("Written to gadget: \n" + this.showStringMap(tmpmap) + "\n");
		}
		catch(Exception exc){this.reportError("Sorry, there was an error while reading the menus state update.\n The Error was logged and will hopefully be removed soon", exc, event);}
	}
   

	
//	Menu section ...
	
	/**
	 * Generate menu.
	 * 
	 * @param The wavelet the robot joins in.
	 */
	public void addMenuToGadget(Event event){
		try{
			BlipContentRefs gadget=event.getWavelet().getRootBlip().first(ElementType.GADGET,Gadget.restrictByUrl(this.getMenuGadgedURL(event.getWavelet())));
			if(gadget.value()!=null ){
				//Create this robots menu
				MenuItem ui = new MenuItem("ui","ui","structure");	
				MenuItem menu = new MenuItem("toolbar","toolbar","toolbar");
				menu.setOption("sort","01");
				ui.appendSubItem(menu);
				
				createMenu(menu, this.getMenuItemFactory(event), event);
		
				//and add it to the gadget
				String oldJson = gadget.value().asElement().getProperty("ui.structure");
				if (oldJson != null) ui.importMenu(new MenuItem(oldJson));
				gadget.updateElement(ImmutableMap.of("ui.structure",ui.toJson()));
				//gadget.updateElement(ui.toMenuState());
				//if (debug) event.getBlip().reply().append("Debug: Menu-JSON \n" + ui.toJson());
				}
			else{
				event.getBlip().reply().append("GADGET NOT FOUND!");				
				}}
		catch(Exception e){this.reportError("Sorry, there was an error while creating the menu gadget.\n The Error was logged and will hopefully be removed soon", e, event);}}
	
	
//	Sequence display section ...
	
	
	
//	Robot comfort section

	/**
	 * Returns the ID the robot has in the current wavelet and generates one if he didn't has one yet.
	 * @param wavelet 
	 * @return
	 * @throws Exception 
	 */
	public String getMyRobotId(Wavelet wavelet) throws JsonParseException{ //%%%%%%
		String id = null;
		if (wavelet.getDataDocuments().contains("ui.robot.id." + this.getRobotName())){
			id = wavelet.getDataDocuments().get("ui.robot.id." + this.getRobotName());}
		else{
			ArrayList<String> blacklist = new ArrayList<String>();
			java.util.Iterator<Entry<String, String>> iter = wavelet.getDataDocuments().iterator();
			while (iter.hasNext()){
				Entry<String, String> entry = iter.next();
				if (entry.getKey().contains("ui.robot.id."))
					blacklist.add(entry.getValue());}
			MenuItemFactory mif = new MenuItemFactory();
			try {
				id = mif.generateID(blacklist);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			wavelet.getDataDocuments().set("ui.robot.id." + this.getRobotName(), id);}
		return id;}
	
	public String showStringMap(Map<String,String> map){
		java.util.Iterator<Entry<String, String>> iter = map.entrySet().iterator();	
		String result = "";
		while (iter.hasNext()){
			Entry<String, String> entry = iter.next();
			result += "- " + entry.getKey() + " : " + entry.getValue() + "\n";}
		return result;
	}
	
	public void connectMenuCallBack(MenuItem menuItem, MenuCallBack menuCallBack){
		if (this.menuCallBacks == null){
			log.warning("menuCallbacks == NULL!");
			menuCallBacks = new HashMap<String,MenuCallBack>();
		}
		this.menuCallBacks.put(menuItem.getID(), menuCallBack);
	}
	
	public String getMenuGadgedURL(Wavelet wavelet){
		return wavelet.getDataDocuments().get("ui.gadget.url");}
	
	public void reportError(String msg, Exception exc, Event event){
		event.getBlip().reply().append(msg);
		log.log(Level.SEVERE, "SBWERROR: " + exc.getMessage(), exc);
		}
}