/*
 *                    BioJava development code
 *
 * This code may be freely distributed and modified under the
 * terms of the GNU Lesser General Public Licence.  This should
 * be distributed with the code.  If you do not have a copy,
 * see:
 *
 *      http://www.gnu.org/copyleft/lesser.html
 *
 * Copyright for this code is held jointly by the individual
 * authors.  These should be listed in @author doc comments.
 *
 * For more information on the BioJava project and its aims,
 * or to join the biojava-l mailing list, visit the home page
 * at:
 *
 *      http://www.biojava.org/
 *
 * Created on 19.03.2004
 * @author Andreas Prlic
 *
 */
package org.synbiowave.biobrick;

import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.Attributes;

import java.util.ArrayList ;
import java.util.HashMap ;
import java.util.List;
import java.util.Map;

// TODO: Auto-generated Javadoc
/**
 * A class to parse the response of a DAS - Feature request.
 * 
 * @author Andreas Prlic
 */
public class BioBrick_DAS_Feature_Handler  extends DefaultHandler{

	/** The features. */
	List<Map<String,String>> features ;
	
	/** The first_flag. */
	boolean first_flag ;
	
	/** The feature. */
	Map<String,String> feature ;
	
	/** The featurefield. */
	String featurefield ;
	
	/** The characterdata. */
	StringBuffer characterdata ;
	
	/** The das command. */
	String dasCommand ;

	/** The come back later. */
	int comeBackLater ;

	/** The max features. */
	int maxFeatures ;

	/** The segment id. */
	String segmentId ;
	
	/** The version. */
	String version;
	
	/** The type_id. */
	String type_id;
	
	/** The type_category. */
	String type_category;
	
	
	/**
	 * Instantiates a new bio brick_ da s_ feature_ handler.
	 * 
	 */
	public BioBrick_DAS_Feature_Handler() {
		super();
		this.features=new ArrayList<Map<String,String>>();
		this.first_flag=true;
		this.featurefield="";
		this.characterdata=new StringBuffer();
		this.dasCommand="";
		this.comeBackLater=-1; 
		this.maxFeatures=-1;
		this.segmentId="";
		this.version="";
		this.type_id="";
		this.type_category="";}
	
	/**
	 * Get the id information specified int the SEGMENT field of the DAS response.
	 * 
	 * @return The segmentId or an emtpy string if not available.
	 * 
	 */
	public String getSegmentId() {return this.segmentId;}

	/**
	 * Get the version informationspecified in the SEGMENT field of the DAS response.
	 * 
	 * @return the version information of an empty string if not available
	 */
	public String getVersion() {return version;}
	
	/**
	 * Checks if passes md5 checksum.
	 * 
	 * @return true, if passed md5 checksum
	 */
	public boolean isMD5Checksum(){
		if((this.version!=null)&&(this.version.length()==32)){return true;}
		return false;}

	/**
	 * Specifies a maximum number of features to be downloaded. If a
	 * server returns more, they will be ignored. Default is to load
	 * all features.
	 * 
	 * @param int maximium number of features to be downloaded.
	 */
	public void setMaxFeatures(int max) {this.maxFeatures=max;}

	/**
	 * Gets max number of features.
	 * 
	 * @return int max number of features.
	 */
	public int getMaxFeatures() {return this.maxFeatures;}

	/**
	 * Sets the dAS command.
	 * 
	 * @param cmd the new dAS command
	 */
	public void setDASCommand(String cmd) {this.dasCommand=cmd;}
	
	/**
	 * Gets the dAS command.
	 * 
	 * @return the dAS command
	 */
	public String getDASCommand() {return this.dasCommand;}

	/**
	 * Gets the _features.
	 * 
	 * @return the _features
	 */
	public List<Map<String,String>> getFeatures() {return this.features;}

	/**
	 * Gets the com back later flag.
	 * 
	 * @return int com back later flag.
	 */
	public int getComBackLater(){return this.comeBackLater;}

	/**
	 * Start feature.
	 * 
	 * @param uri the uri
	 * @param name the name
	 * @param qName the q name
	 * @param atts the atts
	 */
	void start_feature(String uri,String name,String qName,Attributes atts) {
		if((this.maxFeatures>0)&&(this.features.size()>this.maxFeatures)){
			this.characterdata=new StringBuffer();
			return;}
		this.feature=new HashMap<String,String>();
		String id=atts.getValue("id");
		this.feature.put("id",id);
		this.feature.put("dassource",dasCommand);
		this.characterdata=new StringBuffer();}

	/**
	 * Add feature data.
	 * 
	 * @param uri the uri
	 * @param name the name
	 * @param qName the q name
	 */
	void add_featuredata(String uri,String name,String qName) {
		if((this.maxFeatures>0)&&(this.features.size()>this.maxFeatures)){return;}
		String data=this.feature.get(this.featurefield);
		String featureText=this.characterdata.toString();
		if(data!=null){featureText=data+" "+featureText;}
		if(qName.equals("TYPE")){
			if(featureText.length()<1){featureText=this.type_id;}
			this.feature.put("TYPE_ID",type_id);
			this.feature.put("TYPE_CATEGORY", type_category);
			this.type_id="";
			this.type_category="";}
		this.feature.put(this.featurefield,featureText);
		this.featurefield="";
		characterdata=new StringBuffer();}

	/**
	 * Adds the link.
	 * 
	 * @param uri the uri
	 * @param name the name
	 * @param qName the q name
	 * @param atts the atts
	 */
	private void addLink(String uri,String name,String qName,Attributes atts) {
		String href=atts.getValue("href");
		this.feature.put("LINK",href);
		this.characterdata=new StringBuffer();
		this.featurefield="LINK-TEXT";}

	/**
	 * Adds the group.
	 * 
	 * @param uri the uri
	 * @param name the name
	 * @param qName the q name
	 * @param atts the atts
	 */
	private void addGroup(String uri,String name,String qName,Attributes atts) {
		String id=atts.getValue("id");
		this.feature.put("GROUP",id);
		this.characterdata=new StringBuffer();
		this.featurefield="GROUP";}
	
	/* (non-Javadoc)
	 * @see org.xml.sax.helpers.DefaultHandler#startElement(java.lang.String, java.lang.String, java.lang.String, org.xml.sax.Attributes)
	 */
	@Override
	public void startElement(String uri,String name,String qName,Attributes atts){
		if(qName.equals("FEATURE")){start_feature(uri,name,qName,atts);}
		else if(qName.equals("LINK")){addLink(uri,name,qName,atts);}
		else if(qName.equals("GROUP")){addGroup(uri,name,qName,atts);}
		else if(qName.equals("METHOD")||qName.equals("TYPE")||qName.equals("START")||
				qName.equals("END")||qName.equals("NOTE")||qName.equals("SCORE")||qName.equals("ORIENTATION")){
			this.characterdata=new StringBuffer();
			this.featurefield=qName;} 
		else if(qName.equals("SEGMENT")){
			String id=atts.getValue("id");
			if(id!=null){this.segmentId=id;}
			String v=atts.getValue("version");
			if(v!=null){this.version=v;}}
		if(qName.equals("TYPE")){
			this.type_id=atts.getValue("id");
			this.type_category=atts.getValue("category");}}

	/* (non-Javadoc)
	 * @see org.xml.sax.helpers.DefaultHandler#endElement(java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override
	public void endElement(String uri,String name,String qName) {
		if(qName.equals("METHOD")||qName.equals("TYPE")||qName.equals("START")||qName.equals("END")||
				qName.equals("NOTE")||qName.equals("LINK")||qName.equals("SCORE")||qName.equals("ORIENTATION")||
				qName.equals("GROUP")){add_featuredata(uri,name,qName);}
		else if(qName.equals("FEATURE")){
			if(this.maxFeatures>0){
				if(this.features.size()<this.maxFeatures){this.features.add(this.feature);}}
			else{this.features.add(this.feature);}}}

	/* (non-Javadoc)
	 * @see org.xml.sax.helpers.DefaultHandler#characters(char[], int, int)
	 */
	@Override
	public void characters(char ch[],int start,int length){
		if(this.maxFeatures>0){
			if(this.features.size()>this.maxFeatures){return;}}
		for(int i=start;i<start+length;i++){this.characterdata.append(ch[i]);}}}