package org.synbiowave.biobrick;

import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.synbiowave.biojava.SequenceUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/*
 * This Class parses XML Files from MIT Parts Registry http://partsregistry.org
 */


public class PartsRegistryXmlParser {
	
	private static String PartsRegistryLink = "http://partsregistry.org/cgi/xml/part.cgi?part=";
	
	public static BioBrick parseSequenceFromFile(String partID) throws IOException
	{
		//get the factory
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		Document dom = null;
		
		try {

			//Using factory get an instance of document builder
			DocumentBuilder db = dbf.newDocumentBuilder();

			//parse using builder to get DOM representation of the XML file
			dom = db.parse(PartsRegistryLink+partID);


		}catch(ParserConfigurationException pce) {
			pce.printStackTrace();
		}catch(SAXException se) {
			se.printStackTrace();
		}catch(IOException ioe) {
			ioe.printStackTrace();
		}
		
		Element rootElem = dom.getDocumentElement();
		
		// get the partlist
		NodeList nl = rootElem.getElementsByTagName("part");
		
		if(nl == null || nl.getLength() < 1)
		{
			throw(new IOException("Bad file format: No <part> tag detected!"));
		}
		
		// get first part (assuming the file has one part only)
		Element el = (Element)nl.item(0);
		
		// Begin constructing the Sequence
		BioBrick output = new BioBrick();
		
		output.setAuthor(getTextValue(el, "part_author"));
		output.setSequence(readSequenceFromElement(el));
		output.setType(SequenceUtils.determineType(output.getSequence()));
		output.setName(getTextValue(el, "part_name"));
		output.setUrl(getTextValue(el, "part_url"));
		output.setSubmission_date(getTextValue(el, "part_entered"));
		output.setId(getIntValue(el, "part_id"));
		output.setDescr(getTextValue(el, "part_short_desc"));
		
		return output;
	}
	
	private static String readSequenceFromElement(Element el) {
		NodeList nl = el.getElementsByTagName("sequences");
		
		return(getTextValue((Element)nl.item(0),"seq_data").toUpperCase());
	}

	/**
	 * I take a xml element and the tag name, look for the tag and get
	 * the text content
	 * i.e for <employee><name>John</name></employee> xml snippet if
	 * the Element points to employee node and tagName is 'name' I will return John
	 */
	private static String getTextValue(Element ele, String tagName) {
		String textVal = "";
		NodeList nl = ele.getElementsByTagName(tagName);
		if(nl != null && nl.getLength() > 0) {
			Element el = (Element)nl.item(0);
			textVal = el.getFirstChild().getNodeValue();
		}

		return textVal;
	}
	/**
	 * Calls getTextValue and returns a int value
	 */
	private static int getIntValue(Element ele, String tagName) {
		//in production application you would catch the exception
		return Integer.parseInt(getTextValue(ele,tagName));
	}



}
