package org.synbiowave.database;

import java.util.ArrayList;
import java.util.List;

import javax.jdo.PersistenceManager;
import javax.jdo.Query;

import com.google.appengine.api.datastore.Key;
import com.google.wave.api.Annotation;

/** 
 * 
 * This Database Manager can be used for storing robot data in a database configured
 * in the jdoconfig.xml file, which can be found in the META-INF folder of the project.
 * The methods of the class are a very rudiment but useful for the storage of 
 * supplement data like translation tables, restriction site lists or similar data.
 * Mention: The object to store has to be percistible in the means of the jdo protocol.
 * 
 * @author Jonas
 */
public class DBManager {
	
	/**
	 * Stores a generic object into the database.
	 * 
	 * @param The generic object.
	 * 
	 * @return The generic object after storing.
	 * 
	 */
	public static <T> T store(T db_object){
		PersistenceManager pm=PMF.get().getPersistenceManager();
		try{db_object=pm.makePersistent(db_object);}
		finally{pm.close();}
		return db_object;}
	
	/**
	 * Retrieves all objects of a certain generic type which is defined by the
	 * template class. 
	 * 
	 * @param A generic template class.
	 * 
	 * @return A list of the generic objects.
	 * 
	 */
	@SuppressWarnings("unchecked")
	public static <T> List<T> retrieve(T template){
		PersistenceManager pm=PMF.get().getPersistenceManager();
	    List<T> db_objects=(List<T>)pm.newQuery(template.getClass()).execute();
	    if(db_objects.isEmpty()){return null;}
	    else{return db_objects;}}
	
	/**
	 * Retrieves all objects matching the given jdo query.
	 * 
	 * @param A generic template class.
	 * 
	 * @return A list of the generic objects.
	 * 
	 */
	@SuppressWarnings("unchecked")
	public static <T> List<T> getByQuery(Query query){
		List<T> db_objects=null;
		try{db_objects=(List<T>) query.execute();}
	    finally{query.closeAll();}
	    return db_objects;}
	
	/**
	 * Stores a org.synbiowave.workspace.sequence.
	 * 
	 * @param The org.synbiowave.workspace.sequence.
	 * 
	 * @return The org.synbiowave.workspace.sequence after storing.
	 * 
	 */
	public static Key storeSequence(org.synbiowave.workspace.Sequence sequence){
		Sequence db_sequence=new Sequence(sequence.getAuthor(),sequence.getSequence(),sequence.getType());
		db_sequence.setName(sequence.getName());
		for(Annotation annotation:sequence.getAnnotations()){db_sequence.addFeature(new Feature(annotation.getValue(),annotation.getRange().getStart(),annotation.getRange().getEnd(),0));}
		PersistenceManager pm=PMF.get().getPersistenceManager();
		try{db_sequence=pm.makePersistent(db_sequence);}
		finally{pm.close();}
		return db_sequence.getKey();}
	
	/**
	 * Retrieves all org.synbiowave.workspace.sequences. 
	 * 
	 * @param A org.synbiowave.workspace.sequence template.
	 * 
	 * @return A list of all org.synbiowave.workspace.sequences.
	 * 
	 */
	@SuppressWarnings("unchecked")
	public static List<org.synbiowave.workspace.Sequence> getSequences(){
		PersistenceManager pm=PMF.get().getPersistenceManager();
	    List<Sequence> db_sequences=(List<Sequence>)pm.newQuery(Sequence.class).execute();
	    if(db_sequences.isEmpty()){return null;}
	    else{
	    	List<org.synbiowave.workspace.Sequence> sequences=new ArrayList<org.synbiowave.workspace.Sequence>();
	    	for(Sequence db_sequence:db_sequences){
	    		org.synbiowave.workspace.Sequence sequence=new org.synbiowave.workspace.Sequence(db_sequence.getName(),db_sequence.getSequence());
	    		sequence.setAuthor(db_sequence.getAuthor());
	    		sequence.setType(db_sequence.getType());
	    		for(Feature feature:db_sequence.getFeatures()){sequence.addAnnotation(feature.getName(),feature.getStart(),feature.getEnd());}
	    		sequences.add(sequence);}
    	return sequences;}}

	/**
	 * Retrieves a org.synbiowave.workspace.sequence defined by the database key. 
	 * 
	 * @param The database key of the sequence.
	 * 
	 * @return The specified org.synbiowave.workspace.sequence.
	 * 
	 */
	public static org.synbiowave.workspace.Sequence retrieve(Key key){
		PersistenceManager pm=PMF.get().getPersistenceManager();
	    Sequence db_sequence=pm.getObjectById(Sequence.class,key);
	    if(db_sequence!=null){
	    	org.synbiowave.workspace.Sequence sequence=new org.synbiowave.workspace.Sequence(db_sequence.getName(),db_sequence.getSequence());
    		sequence.setAuthor(db_sequence.getAuthor());
    		sequence.setType(db_sequence.getType());
    		for(Feature feature:db_sequence.getFeatures()){sequence.addAnnotation(feature.getName(),feature.getStart(),feature.getEnd());}
    		return sequence;}
	    else{return null;}}}