package org.synbiowave.database;
 
import javax.jdo.annotations.IdGeneratorStrategy;
import javax.jdo.annotations.IdentityType;
import javax.jdo.annotations.PersistenceCapable;
import javax.jdo.annotations.Persistent;
import javax.jdo.annotations.PrimaryKey;

import com.google.appengine.api.datastore.Key;

/** Persitable sequence representation for storing
 * the sequence features in the jdo database.
 * 
 * @author Jonas
 *
 */
@PersistenceCapable(identityType = IdentityType.APPLICATION)
public class Feature {
	
	/** The database key of the feature. */
	@PrimaryKey
    @Persistent(valueStrategy = IdGeneratorStrategy.IDENTITY)
    private Key key;

	/** The sequence the feature belongs to. */
    @Persistent
    private Sequence sequence;

    /** The feature name. */
    @Persistent
    private String name;

    /** The start position of the feature. */
    @Persistent
    private int start;
    
    /** The end position of the feature. */
    @Persistent
    private int end;
    
    /** The orientation of the feature. */
    @Persistent
    private int strand;
	
	/** 
	 * Basic constructor for initializing a sequence annotation.
	 * 
	 * @param String name
	 * @param int start position
	 * @param int end position
	 * @param int strand (negativ=reverse,0=unknown,positiv=forward)
	 * 
	 */
    public Feature(String name,int start,int end,int strand) {
		this.name=name;
		this.start=start;
		this.end=end;
		this.strand=strand;}

	/**
	 * Get the name of the feature.
	 * 
	 * @return String name.
	 * 
	 */
    public String getName(){return name;}

	/**
	 * Set the feature name.
	 * 
	 * @param String name.
	 */
    public void setName(String name){this.name=name;}

    /**
	 * Get the start position of the feature.
	 * 
	 * @return int start position.
	 * 
	 */
    public int getStart(){return start;}

    /**
	 * Set the start position of the feature.
	 * 
	 * @param int start position.
	 */
    public void setStart(int start){this.start=start;}

    /**
	 * Get the end position of the feature.
	 * 
	 * @return int end position.
	 * 
	 */
	public int getEnd(){return end;}

	/**
	 * Set the end position of the feature.
	 * 
	 * @param int end position.
	 */
	public void setEnd(int end){this.end=end;}

	/**
	 * Get the features strand.
	 * 
	 * @return int strand.
	 * 
	 */
	public int getStrand(){return strand;}

	/**
	 * Set the features strand.
	 * 
	 * @param int strand.
	 */
	public void setStrand(int strand){this.strand=strand;}

	/**
	 * Get the database key of this feature.
	 * 
	 * @return Key database key.
	 * 
	 */
	public Key getKey(){return key;}

	/**
	 * Get the sequence the feature belongs to.
	 * 
	 * @return Key database key.
	 * 
	 */
	public Sequence getSequence(){return sequence;}}