package org.synbiowave.database;
 
import java.util.ArrayList;
import java.util.List;

import javax.jdo.annotations.Element;
import javax.jdo.annotations.IdGeneratorStrategy;
import javax.jdo.annotations.IdentityType;
import javax.jdo.annotations.PersistenceCapable;
import javax.jdo.annotations.Persistent;
import javax.jdo.annotations.PrimaryKey;

import com.google.appengine.api.datastore.Key;
import com.google.appengine.api.datastore.Text;

@PersistenceCapable(identityType = IdentityType.APPLICATION)
public class Sequence {
	
	/** The database key of the sequence. */
    @PrimaryKey
    @Persistent(valueStrategy = IdGeneratorStrategy.IDENTITY)
    private Key key=null;

    /** The author. */
    @Persistent
    private String author;
    
    /** The name of the sequence. */
    @Persistent
    private String name;
    
    /** The sequence in aa- or nc-code. */
    @Persistent(defaultFetchGroup="true")
    private Text sequence;
    
    /** The sequence type (DNA,RNA,PROTEIN or PROTEIN-TERM). */
    @Persistent
    private String type;
    
    /** Sequence features. */
    @Persistent(mappedBy = "sequence",defaultFetchGroup = "true")
    @Element(dependent = "true")
    private List<Feature> features;
    
    /** Basic constructor */
    public Sequence(){}
    
    /**
     * Instantiates a new sequence.
     * 
     * @param String author of the sequence.
     * @param String sequence.
     * @param String sequence type.
     */
    public Sequence(String author,String sequence,String type) {
        this.author=author;
        this.name=null;
        this.sequence=new Text(sequence);
        this.type=type;
        this.features=new ArrayList<Feature>();}

	/**
	 * Get the author of the sequence.
	 * 
	 * @return String author.
	 * 
	 */
    public String getAuthor(){return author;}

	/** 
	 * Set the author of the sequence.
	 * 
	 * @param String author.
	 * 
	 */
    public void setAuthor(String author){this.author=author.replaceAll(" ","_");}

    /**
	 * Get the name of the sequence.
	 * 
	 * @return String name.
	 * 
	 */
	public String getName(){return name;}

	/** 
	 * Set the author of the sequence.
	 * 
	 * @param String author.
	 * 
	 */
	public void setName(String name){this.name=name.replaceAll(" ","_");}

	/**
	 * Get the sequence.
	 * 
	 * @return String sequence.
	 * 
	 */
	public String getSequence(){
		if(sequence!=null){return sequence.getValue();}
		else{return null;}}

	/** 
	 * Set the sequence string.
	 * 
	 * @param String sequence.
	 * 
	 */
	public void setSequence(String sequence){this.sequence=new Text(sequence.replaceAll(" ",""));}

	/**
	 * Get the type of the sequence.
	 * 
	 * @return String sequence type.
	 * 
	 */
	public String getType(){return type;}

	/** 
	 * Set the type of the sequence.
	 * 
	 * @param String sequence type.
	 * 
	 */
	public void setType(String type){this.type=type;}
	
	/**
	 * Get a list of the sequence annotations.
	 * 
	 * @return List<Feature> sequence features.
	 * 
	 */
	public List<Feature> getFeatures(){return features;}

	/** 
	 * Add a sequence annotation.
	 * 
	 * @param Feature sequence feature.
	 * 
	 */
	public void addFeature(Feature feature){this.features.add(feature);}
	
	/** 
	 * Add a list of sequence annotations.
	 * 
	 * @param List<Feature> of sequence features.
	 * 
	 */
	public void addFeatures(List<Feature> features){this.features.addAll(features);}
	
	/** 
	 * Set the sequence annotations.
	 * 
	 * @param List<Feature> of sequence features.
	 * 
	 */
	public void setFeatures(List<Feature> features){this.features=features;}

	/**
	 * Get the database key of the sequence.
	 * 
	 * @return Key database key.
	 * 
	 */
	public Key getKey(){return key;}}