/*
    Copyright: synbiowave
     
    License: GPL
     
    Authors: Paul Staab
     
    Version: 0.1 
     
    DESCRIPTION:
    	This class can be used do generate SynBioWave-Menus, to generate JSON-Strings
    	out of it and menus back of JSON-String.
    	Each item can contain options and recusivly other MenuItems later displayed as
    	subitems.
*/

package org.synbiowave.menu;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

import org.synbiowave.menu.items.Formular;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonElement;
import com.google.gson.JsonParseException;

public class MenuItem implements Comparable<MenuItem>
{
	private LinkedHashMap<String, String> options = new LinkedHashMap<String, String>();
	private List<MenuItem> subItemList = new ArrayList<MenuItem>();	

	private Gson gson = null;
	
	protected MenuItem() { }
	
	/**
	 * generates a menu from a JSONObject
	 * @param JSON JSONObject
	 * @throws Exception
	 */
	/*
	public MenuItem(JSONObject JSON) throws Exception 
	{ 
		readJSON(JSON);
	}
	*/
	/**
	 * generates a menu form a JSON-String
	 * @param JSONString JSON-conform String 
	 * @throws Exception
	 */
	public MenuItem(String JSONString) throws JsonParseException 
	{ 
		this.fromJson(JSONString);
	}
	
	/**
	 * generates a menu item
	 * @param type Type of the menu item
	 * @param label Label of the menu item
	 */
	public MenuItem(String type, String label, String key)
	{
		this.options.put("type", type);
		this.options.put("label", label);
		this.options.put("key", key);
	}
	
	//-------------------------------------------------------------------------
	//Getters & Setters
	//-------------------------------------------------------------------------
	
	/**
	 * Returns a saved option of the menu item
	 * @param key option like "type","label" or "key"
	 * @return Value of the option
	 */
	public String getOption(String name)
	{
		return this.options.get(name);
	}
	
	public boolean getBoolOption(String name) throws Exception
	{
		if (this.options.get(name).contentEquals("true")) return true;
		else if (this.options.get(name).contentEquals("false")) return false;
		else throw new Exception("MenuItem-Option '" + name + "' is not boolean.");
	}
	
	public void setOption(String name, String value)
	{
		this.options.put(name, value);
	}
	
	public void setOption(String name, boolean value)
	{
		if (value) this.options.put(name, "true");
		else this.options.put(name, "false");
	}
	
	public LinkedHashMap<String, String> getOptions(String name)
	{
		return this.options;
	}
	
	public void setOptions(LinkedHashMap<String, String> options)
	{
		this.options = options;
	}
	
	public List<MenuItem> getSubItemList()
	{
		return this.subItemList;
	}
	
	public void setSubItemList(List<MenuItem> subItemList)
	{
		this.subItemList = subItemList;
	}
	
	public String getID()
	{
		return this.options.get("key");
	}

	public String getLabel()
	{
		return this.options.get("label");
	}

	
	public LinkedHashMap<String, String> getOptions()
	{
		return this.options;
	}
	
	public MenuItem getSubItemByID(String ID){
		MenuItem returnItem = null;
		for (MenuItem subItem : this.getSubItemList()){
			if (subItem.getID().contentEquals(ID)){
				returnItem = subItem;
			}
			else {
				MenuItem sub2 = subItem.getSubItemByID(ID);
				if (sub2 != null) returnItem = sub2;
			}
		}
		return returnItem;
	}
	
	protected void setParentId(String parentItemId) {
		this.options.put("parentId", parentItemId);;
	}

	public String getParentItemId() {
		return this.options.get("parentId");
	}

	public String getType() {
		return this.options.get("type");
	}
	
	public ArrayList<MenuItem> getSubItemsOfType(String type){
		ArrayList<MenuItem> lists = new ArrayList<MenuItem>();
		for (MenuItem item : this.getSubItemList() ){
			if (item.getType().contentEquals(type)) lists.add(item); 
			}
		return lists;
		}
	
	public String getValue(){
		return this.getOption("value");
	}

	public MenuItem getTopForm(MenuItem globalMenu) {
		MenuItem form = null;
		
		if ( this.getParentItemId() != null ){
			MenuItem topItem = globalMenu.getSubItemByID(this.getParentItemId());
			if ( topItem.getType().contentEquals("form") )
				form = topItem;
			else
				form = topItem.getTopForm(globalMenu);}
		
		return form;
	}
	
	private Gson getGson(){
		if (this.gson == null){
			GsonBuilder gson = new GsonBuilder();
			gson.registerTypeAdapter(MenuItem.class, new MenuItemSerializer());
			gson.registerTypeAdapter(Formular.class, new MenuItemSerializer());
			gson.registerTypeAdapter(MenuItem.class, new MenuItemDeserializer());
			this.gson = gson.serializeNulls().create();}
		return this.gson;
	}
	
	//-------------------------------------------------------------------------
	//Public functions
	//-------------------------------------------------------------------------
		
	/**
	 * Adds a MenuItem as a SubItem
	 * @param menuItem the MenuItem a append
	 * @return this
	 */
	public MenuItem appendSubItem(MenuItem menuItem)
	{
		this.subItemList.add(menuItem);
		menuItem.setParentId(this.getID());
		return this;
	}

	public void removeSubItemByID(String id) throws Exception {
		MenuItem subItem = null;
		for (MenuItem subItem2 : this.getSubItemList()){
			if (subItem2.getID().contentEquals(id)) subItem = subItem2;}
		if (subItem != null){
			this.subItemList.remove(subItem);}
		else {
			throw new Exception("Subitem to remove not found");}
	}
	
	/**
	 * Removes all Sub-MenuItems
	 */
	public void removeAllSubItems()
	{
		subItemList.removeAll(subItemList);	
	}
	
	/**
	 * Adds all Sub-MenuItem of menu to this Menu Item
	 * @param menu
	 */
	public void importMenu(MenuItem menu)
	{
		for (MenuItem importItem : menu.getSubItemList()) 
		{
			boolean exists = false;
			for (MenuItem exisitingItem : this.getSubItemList())
			{
				if (importItem.getOption("label").contentEquals(exisitingItem.getOption("label")))
				{
					exisitingItem.importMenu(importItem);
					exists = true;
					continue;
				}		
			}
			if ( ! exists )
			{
				this.appendSubItem(importItem);
			}	
		}
	}
		
	public String toJson(){
		return this.getGson().toJson(this); //.replaceAll("\"", "'");
		}
	
	public void fromJson(String jsonstring){
		MenuItem mi = this.getGson().fromJson(jsonstring, MenuItem.class);
		this.options = mi.getOptions();
		this.subItemList = mi.getSubItemList();
	}
	
	/* For later use
	public LinkedHashMap<String,String> toMenuState()
	{
		java.util.Collections.sort(this.subItemList);
		
		LinkedHashMap<String,String> state = new LinkedHashMap<String,String>();
		state.put("ui." + this.getOption("key"), this.toJson());
		
		for (MenuItem subItem : this.getSubItemList()){
			state.putAll(subItem.toMenuState());}
		
		return state;
	}
	*/
	
	@Override
	public int compareTo(MenuItem o) {
		if ( o.getOptions().containsKey("sort") && this.getOptions().containsKey("sort"))
			return (-o.getOption("sort").compareTo(this.getOption("sort")));
		else 
			return 0;
	}
	
	protected void sortSubItems(){
		java.util.Collections.sort(this.subItemList);
	}
	
	protected JsonElement toJsonElement(){
		return this.getGson().toJsonTree(this);
		}
}