package org.synbiowave.menu.items;

import java.util.ArrayList;
import java.util.HashMap;

import org.synbiowave.AbstractSynBioWaveRobot;
import org.synbiowave.menu.MenuItem;
import org.synbiowave.menu.MenuItemFactory;

import com.google.appengine.repackaged.com.google.common.collect.ImmutableMap;
import com.google.gson.JsonParseException;
import com.google.wave.api.BlipContentRefs;
import com.google.wave.api.ElementType;
import com.google.wave.api.Gadget;
import com.google.wave.api.event.Event;

public class Formular extends MenuItem {

	private int counter = 1;
	private MenuItem form = null;
	private String key = null;
	
	private String generateSubItemID(){
		return this.key + "i" + counter++;}
	
	@Override
	public String getID(){
		return key + "f";
	}
	
	public Formular(String JSON) throws JsonParseException{
		super(JSON);
	}
	
	public Formular(MenuItem menuItem) throws JsonParseException{
		super();
		if ( !menuItem.getOption("type").contentEquals("button") ||
			 menuItem.getSubItemList().size() != 1 ||
			 !menuItem.getSubItemList().get(0).getOption("type").contentEquals("form") )
			throw new IllegalArgumentException("conversion of MenuItem to Formular failed;\n" + 
					"JSON:\n" + menuItem.toJson() + "\n\n" + 
					menuItem.getOption("type") + " == button ?\n" +
					Integer.toString(menuItem.getSubItemList().size()) + " == 1 ?\n" +
					menuItem.getSubItemList().get(0).getOption("type") + " == form ?\n" );
		else{
			this.setOptions(menuItem.getOptions());
			this.setSubItemList(menuItem.getSubItemList());
			this.key = menuItem.getID();
			this.form = menuItem.getSubItemList().get(0);}
	}
	
	public Formular(String label, String heading, String key) throws JsonParseException {
		super("button",label,key);
		this.key = key;
		this.form = new MenuItem("form", heading, key + "f");
		this.appendSubItem(form);}
	
	
	public void setOption(String key,String value){
		super.setOption(key, value);
		form.setOption(key, value);
	}
	
	public String getLableOfSelectedRatio(String group){
		String label = "";
		for (MenuItem rad : this.getPureForm().getSubItemsOfType("radio")){	
			if (rad.getOption("group").contentEquals(group) && rad.getOption("value").contentEquals("true")){
				label = rad.getOption("label");
			}
		}
		return label;
	}
	
	public HashMap<String,Boolean> getCheckSelection(String group) throws Exception{
		HashMap<String,Boolean> map = new HashMap<String,Boolean>();
		for (MenuItem rad : this.getPureForm().getSubItemsOfType("checkbox")){	
			if (rad.getOption("group").contentEquals(group)){
				map.put(rad.getLabel(), rad.getBoolOption("value"));
			}
		}
		return map;
	}
	
	public MenuItem getPureForm(){
		return this.form;}
	
	public MenuItem getButton(){
		return this;}
	
	public void addElement(MenuItem menuItem){
		this.form.appendSubItem(menuItem);}
	
	@Override
	public ArrayList<MenuItem> getSubItemsOfType(String type){
		return this.getPureForm().getSubItemsOfType(type);
	}
	
	public MenuItem addUpload(String label, String targetURL, String uploadID, String order){
		MenuItem Upload = new MenuItem("upload",label,generateSubItemID());
		Upload.setOption("target", targetURL);
		Upload.setOption("enctype", "multipart/form-data");
		Upload.setOption("uploadid", uploadID);
		Upload.setOption("sort", order);
		this.form.appendSubItem(Upload);
		return Upload;
		}
		
	public void addDownload(String label, String targetURL, String downloadID, String order){
		MenuItem download = new MenuItem("download",label,generateSubItemID());
		download.setOption("target", targetURL);
		download.setOption("enctype", "multipart/form-data");
		download.setOption("uploadid", downloadID);
		download.setOption("sort", order);
		this.form.appendSubItem(download);}

	public void addRadioButton(String label, String group, String order) {
		MenuItem radio = new MenuItem("radio", label, generateSubItemID());
		radio.setOption("group", group);
		radio.setOption("sort", order);
		this.form.appendSubItem(radio);}
	
	public void addCheckButton(String label, String group, String order) {
		MenuItem radio = new MenuItem("checkbox", label, generateSubItemID());
		radio.setOption("group", group);
		radio.setOption("sort", order);
		this.form.appendSubItem(radio);}
	
	public void addInputfield(String label, String defaultValue, String order){
		MenuItem input = new MenuItem("textfield",label,generateSubItemID());
		input.setOption("sort", order);
		input.setOption("value", defaultValue);
		this.form.appendSubItem(input);}
	
	public MenuItem addList(String label, String order){
		MenuItem list = new MenuItem("list", label, generateSubItemID());
		list.setOption("sort", order);
		this.form.appendSubItem(list);
		return list;}
	
	public MenuItem addListitemToList(MenuItem list, String label, String value){
		MenuItem listitem = new MenuItem("listitem",label,generateSubItemID());
		listitem.setOption("value", value);
		list.appendSubItem(listitem);
		return listitem;}
	
	public ArrayList<MenuItem> getLists(){
		return this.form.getSubItemsOfType("list");
	}
	
	public void addFormToCompleteMenu(Event event, AbstractSynBioWaveRobot servlet) throws Exception{
		MenuItem ui=new MenuItem("ui","ui","ui");
		MenuItemFactory mif=servlet.getMenuItemFactory(event);
		mif.setRobotID(servlet.getMyRobotId(event.getWavelet()));
		
		ui.appendSubItem(this.getPureForm());

		BlipContentRefs gadget=event.getWavelet().getRootBlip().first(ElementType.GADGET,Gadget.restrictByUrl(servlet.getMenuGadgedURL(event.getWavelet())));
		if(gadget.value()!=null){
			String oldJson=gadget.value().asElement().getProperty("ui.structure");
			if(oldJson!=null){
				ui.importMenu(new MenuItem(oldJson));
			}
			gadget.updateElement(ImmutableMap.of("ui.structure",ui.toJson()));
		}
	}
}
