package org.synbiowave.workspace;

import java.util.HashMap;

import org.synbiowave.workspace.Sequence;

public class RichSequence extends Sequence {
	
	public String prefix;
	static public HashMap<Character, Character> DNAComplement = new HashMap<Character, Character>();
	static
	{
		DNAComplement.put('A', 'T');
		DNAComplement.put('T', 'A');
		DNAComplement.put('C', 'G');
		DNAComplement.put('G', 'C');
	}
	
	public RichSequence(Sequence input)
	{		
		this.setAuthor(input.getAuthor());
		this.setName(input.getName());
		this.setSequence(input.getSequence());
		this.setType(input.getType());
		
		this.prefix = new String(""); 
	}
	
	public RichSequence()
	{
	}
	
	
	public boolean isDNA()
	{
		for(int i = 0; i < getSequence().length(); i++)
		{
			char c = getSequence().charAt(i);
			c = Character.toLowerCase(c);
			if(c != 'a' && c != 't' && c != 'c' && c != 'g') return false;
		}
		return true;
	}
	
	// Returns the Complement of the Sequence
	public RichSequence getComplement()
	{
		RichSequence output = new RichSequence();
		output.setAuthor(this.getAuthor());
		output.setName(this.getName() + "(Complement");

		StringBuilder Comp = new StringBuilder();
		String original = this.getSequence();
		for(int i = 0; i < original.length(); i++)
		{
			Comp.append(DNAComplement.get(original.charAt(i)));
		}
		output.setSequence(Comp.toString());
		
		return output;
		
	}
	
	public RichSequence ReverseComplement()
	{
		RichSequence output = getComplement();
		output.Reverse();
		
		return output;
	}
	
	public void Reverse()
	{
		setSequence(new StringBuffer(getSequence()).reverse().toString());
	}
	
	/* 
	 * Returns the meltingTemperature of the Sequence
	 * based on the formulas found here:
	 * http://www.basic.northwestern.edu/biotools/oligocalc.html
	 */
	public double getMeltingPoint(boolean ignorePrefix)
	{
		String sequence;
		
		if(ignorePrefix)
		{
			sequence = getSequence();
		}else
		{
			sequence = getPrefix() + getSequence();
		}
		
		return getMeltingPoint(sequence);
	}

	public static double getMeltingPoint(String sequence) {
		int amountAT = 0;
		int amountCG = 0;
		
		for(int i = 0; i < sequence.length(); i++)
		{
			char c = sequence.charAt(i);
			
			c = Character.toLowerCase(c);
			
			if(c == 'a' || c == 't') amountAT++;
			if(c == 'g' || c == 'c') amountCG++;
		}
		
		// Choose formula
		if(sequence.length() > 13)
		{
			return(getComplexMeltingPoint(amountAT, amountCG));
		}else
		{
			return(getSimpleMeltingPoint(amountAT, amountCG));
		}
	}
	
	/*
	 * Calculates the Meltingpoint of a given Sequence using
	 * T = 64 + 41*(G+C-16.4)/(A + T + C + G)
	 */
	private static double getComplexMeltingPoint(int amountAT, int amountCG) {

		if (amountCG == 0 && amountAT == 0)return 0;
		
		return( 64 + 41*((amountCG - 16.4)/(amountCG + amountAT)));
	}

	/*
	 * Calculates the Meltingpoint of a given Sequence using
	 * T = 2 * CG + 4 * AT
	 */
	private static int getSimpleMeltingPoint(int amountAT, int amountCG)
	{
		return(amountAT * 2 + amountCG * 4);
		
	}
	public boolean hasPrefix()
	{
		return(prefix.length() != 0);
	}
	
	public void setPrefix(String prefix)
	{
		// TODO: should check if prefix is valid, i.e. DNA
		this.prefix = prefix;
	}
	
	public String getPrefix()
	{
		return prefix;
	}
	
	
	public static String generateLeadingGaps(int number)
	{
		StringBuilder gaps = new StringBuilder("");
		for(int i = 0; i < number; i++)
		{
			gaps.append("-");
		}
		return gaps.toString();
		
	}

}